package cn.dankal.test.boot.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.listener.DeadLetterPublishingRecoverer;
import org.springframework.kafka.listener.ErrorHandler;
import org.springframework.kafka.listener.SeekToCurrentErrorHandler;

/**
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-29
 */
@Configuration
public class KafkaBaseConfig {

    /**
     * retryCount
     */
    @Value("${spring.kafka.retry-count:3}")
    private Integer retryCount;


    /**
     * 配置 {@link SeekToCurrentErrorHandler} 用于消息消费重试
     *
     * @param template KafkaTemplate
     * @return ErrorHandler
     * @see DeadLetterPublishingRecoverer
     */
    @Bean
    @Primary
    public ErrorHandler kafkaErrorHandler(KafkaTemplate<Object, Object> template) {
        DeadLetterPublishingRecoverer deadLetterPublishingRecoverer = new DeadLetterPublishingRecoverer(template);
        return new SeekToCurrentErrorHandler(deadLetterPublishingRecoverer, retryCount);
    }
}
