package cn.dankal.test.boot.controller;

import cn.dankal.test.boot.common.Const;
import cn.dankal.test.boot.entity.DankalSimpleMessage;
import cn.dankal.test.boot.factory.ThreadExecutorFactory;
import cn.dankal.test.boot.service.KafkaDankalMessageService;
import cn.dankal.test.boot.service.KafkaStringMessageService;
import cn.hutool.core.util.RandomUtil;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.concurrent.CyclicBarrier;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-28
 */
@RestController
public class SendMessageController {

    /**
     * KafkaMessageService
     */
    private final KafkaDankalMessageService kafkaDankalMessageService;

    /**
     * KafkaStringMessageService
     */
    private final KafkaStringMessageService kafkaStringMessageService;

    /**
     * ThreadPoolExecutor
     */
    private final static ThreadPoolExecutor THREAD_POOL_EXECUTOR = ThreadExecutorFactory.getThreadPoolExecutor("dankal-test-spring-kafka");

    public SendMessageController(KafkaDankalMessageService kafkaDankalMessageService, KafkaStringMessageService kafkaStringMessageService) {
        this.kafkaDankalMessageService = kafkaDankalMessageService;
        this.kafkaStringMessageService = kafkaStringMessageService;
    }

    @GetMapping("/hello/simple/{count}")
    public String sendHelloMessageBySimple(@PathVariable("count") Integer count) {
        final String topic = Const.Kafka.Topic.TOPIC_HELLO_SPRING_KAFKA;
        if (count <= 0) {
            count = 10;
        }
        for (int i = 0; i < count; i++) {
            final String key = "dankal-test-" + RandomUtil.randomInt(1, 10000);
            DankalSimpleMessage dankalSimpleMessage = new DankalSimpleMessage()
                    .setId(i + 1)
                    .setKey(key)
                    .setContent(RandomUtil.randomString(Const.RANDOM_BASE_STRING, 15))
                    .setSendTime(LocalDateTime.now());
            try {
                kafkaDankalMessageService.sendMessageBySimple(topic, key, dankalSimpleMessage);
            } catch (Exception e) {
                e.printStackTrace();
                return "error";
            }
        }
        return "success";
    }

    @GetMapping("/hello/sync/{count}")
    public String sendHelloMessageBySync(@PathVariable("count") Integer count) {
        final String topic = Const.Kafka.Topic.TOPIC_HELLO_SPRING_KAFKA;
        if (count <= 0) {
            count = 10;
        }
        for (int i = 0; i < count; i++) {
            final String key = "dankal-test-" + RandomUtil.randomInt(1, 10000);
            DankalSimpleMessage dankalSimpleMessage = new DankalSimpleMessage()
                    .setId(i + 1)
                    .setKey(key)
                    .setContent(RandomUtil.randomString(Const.RANDOM_BASE_STRING, 15))
                    .setSendTime(LocalDateTime.now());
            try {
                kafkaDankalMessageService.sendMessageBySync(topic, key, dankalSimpleMessage);
            } catch (Exception e) {
                e.printStackTrace();
                return "error";
            }
        }
        return "success";
    }

    @GetMapping("/hello/async/{count}")
    public String sendHelloMessageByAsync(@PathVariable("count") Integer count) {
        final String topic = Const.Kafka.Topic.TOPIC_HELLO_SPRING_KAFKA;
        if (count <= 0) {
            count = 10;
        }
        for (int i = 0; i < count; i++) {
            final String key = "dankal-test-" + RandomUtil.randomInt(1, 10000);
            DankalSimpleMessage dankalSimpleMessage = new DankalSimpleMessage()
                    .setId(i + 1)
                    .setKey(key)
                    .setContent(RandomUtil.randomString(Const.RANDOM_BASE_STRING, 15))
                    .setSendTime(LocalDateTime.now());
            try {
                kafkaDankalMessageService.sendMessageByAsync(topic, key, dankalSimpleMessage);
            } catch (Exception e) {
                e.printStackTrace();
                return "error";
            }
        }
        return "success";
    }


    @GetMapping("/test/string")
    public String testSendStringMessage() {
        final String topic = Const.Kafka.Topic.TOPIC_SIMPLE_STRING_MESSAGE;
        final String key = "dankal-test-str-" + RandomUtil.randomInt(10, 500);
        final String message = RandomUtil.randomString(20);
        kafkaStringMessageService.sendStringMessage(topic, key, message);
        return "success";
    }

    @GetMapping("/string/batch/{count}")
    public String testBatchSendMessage(@PathVariable("count") Integer batchCount) {
        if (batchCount <= 0) {
            batchCount = 10;
        }
        CyclicBarrier cyclicBarrier = new CyclicBarrier(batchCount);
        for (int i = 0; i < batchCount; i++) {
            THREAD_POOL_EXECUTOR.submit(() -> {
                final String topic = Const.Kafka.Topic.TOPIC_SIMPLE_STRING_MESSAGE;
                final String key = "dankal-test";
                final String message = RandomUtil.randomString(20);
                try {
                    // 保证一起发送
                    cyclicBarrier.await();
                    kafkaStringMessageService.sendStringMessage(topic, key, message);
                } catch (Exception e) {
                    //ignored
                }
            });
        }
        return "success";
    }

}
