package cn.dankal.test.boot.listener;

import cn.dankal.test.boot.common.Common;
import cn.dankal.test.boot.common.Const;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-29
 */
@Slf4j
@Component
@ConditionalOnProperty(prefix = "spring.kafka",
        name = {"listener.type"}, havingValue = "batch")
public class BatchStringKafkaMessageListener {

    @KafkaListener(
            topics = {Const.Kafka.Topic.TOPIC_SIMPLE_STRING_MESSAGE},
            groupId = Const.Kafka.ConsumerGroupId.GROUP_ID_SIMPLE_STRING_CONSUMER
    )
    public void batchHandlerStringMessage(List<String> messageList, Acknowledgment acknowledgment) {
        if (log.isDebugEnabled()) {
            log.debug(
                    "{} 成功获取到 {} 条数据",
                    Const.FULL_DATE_TIME_FORMATTER.format(LocalDateTime.now()),
                    messageList.size()
            );
        }
        for (String message : messageList) {
            if (log.isInfoEnabled()) {
                log.info(
                        "{} {} 成功消费消息: {}",
                        Const.FULL_DATE_TIME_FORMATTER.format(LocalDateTime.now()),
                        Common.getSpringKafkaConsumerName(Thread.currentThread().getName()),
                        message);
            }
        }
        acknowledgment.acknowledge();
    }
}
