package cn.dankal.test.boot.listener;

import cn.dankal.test.boot.common.Common;
import cn.dankal.test.boot.common.Const;
import cn.dankal.test.boot.handler.KafkaStringMessageListenerErrorHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.handler.annotation.SendTo;

import java.time.LocalDateTime;

/**
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-29
 */
@Slf4j
//@Component("testErrorHandler")
public class KafkaListenerErrorHandlerTestListener {

    /**
     * <pre>
     *     此监听器用来测试 spring kafka 对 消息消费异常的处理机制
     *     此监听器接收的 String 类型的 message,但是其订阅 主题 所传递的消息是: DankalMessage
     *     故将会抛出异常,以达到测试 errorHandler 的作用
     * </pre>
     *
     * @param message        需要消息消息
     * @param acknowledgment 消息确认器
     * @return 异常消息
     */
    @KafkaListener(
            topics = {Const.Kafka.Topic.TOPIC_HELLO_SPRING_KAFKA},
            groupId = Const.Kafka.ConsumerGroupId.GROUP_ID_TEST_STRING_MESSAGE_HANDLER_ERROR_CONSUMER,
            errorHandler = "kafkaStringMessageListenerErrorHandler"
    )
    @SendTo({Const.Kafka.Topic.TOPIC_HANDLER_STRING_MESSAGE_ERROR_MESSAGE})
    public String testStringMessageHandlerError(String message, Acknowledgment acknowledgment) {
        if (log.isInfoEnabled()) {
            log.info("{} 执行线程: {} 消费消息 ==> 主题: {}",
                    Const.FULL_DATE_TIME_FORMATTER.format(LocalDateTime.now()),
                    Common.getSpringKafkaConsumerName(Thread.currentThread().getName()),
                    message);
        }
        acknowledgment.acknowledge();
        return "handler message: " + message + " success";
    }

    /**
     * <pre>
     * 此监听器 是用来处理 {@link #testStringMessageHandlerError(String, Acknowledgment)} 消费消息时出现异常时,
     * {@link KafkaStringMessageListenerErrorHandler} 处理方法的方法返回值.
     * </pre>
     *
     * @param message        异常处理方法返回值
     * @param acknowledgment 消息确认器
     */
    @KafkaListener(
            topics = {Const.Kafka.Topic.TOPIC_HANDLER_STRING_MESSAGE_ERROR_MESSAGE},
            groupId = Const.Kafka.ConsumerGroupId.GROUP_ID_TEST_STRING_MESSAGE_HANDLER_ERROR_CONSUMER
    )
    public void handlerStringMessageError(String message, Acknowledgment acknowledgment) {
        if (log.isInfoEnabled()) {
            log.info("handlerStringMessageError ==> {} 执行线程: {} 处理异常信息 ==> 异常消息: {}",
                    Const.FULL_DATE_TIME_FORMATTER.format(LocalDateTime.now()),
                    Common.getSpringKafkaConsumerName(Thread.currentThread().getName()),
                    message);
        }
        acknowledgment.acknowledge();
    }
}
