package cn.dankal.test.boot.service.impl;

import cn.dankal.test.boot.component.AsyncSendMessageFailureCallback;
import cn.dankal.test.boot.component.DankalMessageSuccessCallback;
import cn.dankal.test.boot.entity.DankalSimpleMessage;
import cn.dankal.test.boot.service.KafkaDankalMessageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.stereotype.Service;

import java.util.concurrent.ExecutionException;

/**
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-21
 */
@Slf4j
@Service
public class KafkaDankalMessageServiceImpl implements KafkaDankalMessageService {

    /**
     * KafkaTemplate<String, DankalSimpleMessage>
     */
    private final KafkaTemplate<String, DankalSimpleMessage> kafkaTemplate;

    /**
     * DankalMessageSuccessCallback
     */
    private final DankalMessageSuccessCallback dankalMessageSuccessCallback;

    /**
     * DankalMessageFailureCallback
     */
    private final AsyncSendMessageFailureCallback asyncSendMessageFailureCallback;

    public KafkaDankalMessageServiceImpl(KafkaTemplate<String, DankalSimpleMessage> kafkaTemplate, DankalMessageSuccessCallback dankalMessageSuccessCallback, AsyncSendMessageFailureCallback asyncSendMessageFailureCallback) {
        this.kafkaTemplate = kafkaTemplate;
        this.dankalMessageSuccessCallback = dankalMessageSuccessCallback;
        this.asyncSendMessageFailureCallback = asyncSendMessageFailureCallback;
    }

    @Override
    public boolean sendMessageBySimple(String topic, String key, DankalSimpleMessage dankalSimpleMessage) {
        // 发送并忘记
        kafkaTemplate.send(topic, key, dankalSimpleMessage);
        return true;
    }

    @Override
    public boolean sendMessageBySync(String topic, String key, DankalSimpleMessage dankalSimpleMessage) throws ExecutionException, InterruptedException {
        // 同步发送
        SendResult<String, DankalSimpleMessage> sendResult = kafkaTemplate.send(topic, key, dankalSimpleMessage).get();
        return sendResult.getRecordMetadata().offset() > 0;
    }

    @Override
    public boolean sendMessageByAsync(String topic, String key, DankalSimpleMessage dankalSimpleMessage) {
        // 异步发送
        kafkaTemplate
                .send(topic, key, dankalSimpleMessage)
                // 添加异步回调处理
                .addCallback(dankalMessageSuccessCallback, asyncSendMessageFailureCallback);
        return true;
    }
}
