package cn.dankal.test.cloud.producer.controller;

import cn.dankal.test.cloud.producer.entity.DankalSimpleMessage;
import cn.dankal.test.cloud.producer.output.KafkaDankalMessageOutput;
import cn.hutool.core.util.RandomUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-02-01
 */
@Slf4j
@RestController
public class SendMessageController {

    /**
     * KafkaDankalMessageOutput
     */
    private final KafkaDankalMessageOutput kafkaDankalMessageOutput;


    public SendMessageController(KafkaDankalMessageOutput kafkaDankalMessageOutput) {
        this.kafkaDankalMessageOutput = kafkaDankalMessageOutput;
    }

    @GetMapping("/json/{count}")
    public String sendAssignCountMessage(@PathVariable("count") Integer count) {
        if (count <= 0) {
            count = 10;
        }
        for (int i = 0; i < count; i++) {
            final String key = "dankal-cloud-kafka-" + RandomUtil.randomInt(100, 100000);
            final DankalSimpleMessage dankalSimpleMessage = new DankalSimpleMessage()
                    .setId(RandomUtil.randomInt(10, 1000000))
                    .setContent(RandomUtil.randomString(15))
                    .setSendTime(LocalDateTime.now())
                    .setKey(key);
            Message<DankalSimpleMessage> dankalSimpleMessageMessage = MessageBuilder.withPayload(dankalSimpleMessage).build();
            kafkaDankalMessageOutput.kafkaDankalMessageOutput()
                    .send(dankalSimpleMessageMessage);
        }
        return String.valueOf(count);
    }

    @GetMapping("/condition/{count}")
    public String sendAssignCountConditionMessage(@PathVariable("count") Integer count) {
        if (count <= 0) {
            count = 10;
        }
        int setHeaderTagCount = 0;
        for (int i = 0; i < count; i++) {
            final String key = "dankal-cloud-kafka-" + RandomUtil.randomInt(100, 100000);
            final DankalSimpleMessage dankalSimpleMessage = new DankalSimpleMessage()
                    .setId(RandomUtil.randomInt(10, 1000000))
                    .setContent(RandomUtil.randomString(15))
                    .setSendTime(LocalDateTime.now())
                    .setKey(key);
            MessageBuilder<DankalSimpleMessage> messageBuilder = MessageBuilder
                    .withPayload(dankalSimpleMessage);
            if (RandomUtil.randomBoolean()) {
                // 随机添加 message  header tag
                messageBuilder.setHeader("messageTag", "dankal-condition");
                setHeaderTagCount++;
            }
            Message<DankalSimpleMessage> dankalSimpleMessageMessage = messageBuilder
                    .build();
            kafkaDankalMessageOutput.kafkaDankalMessageOutput()
                    .send(dankalSimpleMessageMessage);
        }
        log.info("共发送 {} 条消息,其中 {} 条 带有 header tag", count, setHeaderTagCount);
        return String.valueOf(count);
    }
}
